--[[
Copyright (C) Achimobil, 2022

Author: Achimobil
Date: 09.03.2022
Version: 1.0.0.0

Contact:
https://forum.giants-software.com
https://discord.gg/Va7JNnEkcW

History:
V 1.0.0.0 @ 03.03.2022 - First Release

Important:
No copy and use in own mods allowed.
]]

TensionbeltsFix = {}

TensionbeltsFix.metadata = {
    title = "TensionbeltsFix",
    notes = "Just a fix for the scrap Giants has done",
    author = "Achimobil",
    info = "No copy and use in own mods allowed."
};
TensionbeltsFix.modDir = g_currentModDirectory;

--- Original from Source 1.2.0.2
function TensionbeltsFix:freeTensionBeltObject(_, objectId, objectsToJointTable, isDynamic, object)
-- print("test")
	if entityExists(objectId) then
		local jointData = objectsToJointTable[objectId]

		if isDynamic then
			if self.isServer and jointData ~= nil then
				removeJoint(jointData.jointIndex)
				delete(jointData.jointTransform)

				if object ~= nil then
					if object.setReducedComponentMass ~= nil then
						object:setReducedComponentMass(false)
						self:setMassDirty()
					end

					if object.setCanBeSold ~= nil then
						object:setCanBeSold(true)
					end
				end
			end
		else
			local parentNode = nil

			if jointData ~= nil then
				parentNode = jointData.parent
			end

			if parentNode ~= nil and not entityExists(parentNode) then
				delete(objectId)
			else
				if parentNode == nil then
					parentNode = getRootNode()
				end

				local x, y, z = getWorldTranslation(objectId)
				local rx, ry, rz = getWorldRotation(objectId)

				if object ~= nil and object.unmountKinematic ~= nil then
					object:unmountKinematic()
				elseif self.isServer then
					link(parentNode, objectId)
					setWorldTranslation(objectId, x, y, z)
					setWorldRotation(objectId, rx, ry, rz)
					setRigidBodyType(objectId, RigidBodyType.DYNAMIC)
				end
			end
		end
	end

	objectsToJointTable[objectId] = nil
end

TensionBelts.freeTensionBeltObject = Utils.overwrittenFunction(TensionBelts.freeTensionBeltObject, TensionbeltsFix.freeTensionBeltObject)

function TensionbeltsFix:lockTensionBeltObject(_, objectId, objectsToJointTable, isDynamic, jointNode, object)
-- print("test2")
	if objectsToJointTable[objectId] == nil then
		if isDynamic then
			if self.isServer then
				local constr = JointConstructor.new()

				constr:setActors(jointNode, objectId)

				local jointTransform = createTransformGroup("tensionBeltJoint")

				link(self.spec_tensionBelts.linkNode, jointTransform)

				local x, y, z = localToWorld(objectId, getCenterOfMass(objectId))

				setWorldTranslation(jointTransform, x, y, z)
				constr:setJointTransforms(jointTransform, jointTransform)
				constr:setRotationLimit(0, 0, 0)
				constr:setRotationLimit(1, 0, 0)
				constr:setRotationLimit(2, 0, 0)

				local springForce = 1000
				local springDamping = 10

				constr:setRotationLimitSpring(springForce, springDamping, springForce, springDamping, springForce, springDamping)
				constr:setTranslationLimitSpring(springForce, springDamping, springForce, springDamping, springForce, springDamping)

				local jointIndex = constr:finalize()

				if object ~= nil then
					if object.setReducedComponentMass ~= nil then
						object:setReducedComponentMass(true)
						self:setMassDirty()
					end

					if object.setCanBeSold ~= nil then
						object:setCanBeSold(false)
					end
				end

				objectsToJointTable[objectId] = {
					jointIndex = jointIndex,
					jointTransform = jointTransform,
					object = object
				}
			end
		else
			local parentNode = getParent(objectId)
			local x, y, z = localToLocal(objectId, jointNode, 0, 0, 0)
			local rx, ry, rz = localRotationToLocal(objectId, jointNode, 0, 0, 0)

			if object ~= nil and object.mountKinematic ~= nil then
				object:mountKinematic(self, jointNode, x, y, z, rx, ry, rz)
			elseif self.isServer then
				setRigidBodyType(objectId, RigidBodyType.KINEMATIC)
				link(jointNode, objectId)
				setTranslation(objectId, x, y, z)
				setRotation(objectId, rx, ry, rz)
			end

			objectsToJointTable[objectId] = {
				parent = parentNode,
				object = object
			}
		end
	end
end


TensionBelts.lockTensionBeltObject = Utils.overwrittenFunction(TensionBelts.lockTensionBeltObject, TensionbeltsFix.lockTensionBeltObject)







